//+-------------------------------------------------------------+
//|                                                MTCF_V4a.mq4 |
//|V1a          Comment Range, Alert Buy - Sell                 |
//|V2a          Expand Alert Buy - Sell                         |
//|V2b          NYClose extern variable added                   |
//|V3a          Restart added, Alerts modified                  |
//|V3e          Alerts modified, comment preserved              |
//|V3f          Restart test moved                              |
//|V3g          Alerts modified                                 |
//|V4a          Restart = True after trade ONLY                 |
//+-------------------------------------------------------------+
//---- input parameters
extern int        RangeStart=4;
extern int        RangeEnd=6;
extern int        NYClose=21;
extern bool       UseMoneyManagement = true;
extern double     Risk=1.5;
extern double     Lots = 1;
extern int        MagicNumber = 1;
extern int        Slippage = 3;
extern bool       BrokerECN = true;
extern bool       Broker5Digits = false;
extern bool       ShowRectangle = false;
extern bool       Restart  =  true;    //true = no trade till new day.  false - trade anytime

datetime   bartime = 0;
datetime lasttrade = 0;
int        SL = 15;
int        TP = 45;
int SLMoved = 0;
int k = 1;

int HH, LL;
double HighestHigh, LowestLow;
//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
{
//----
   if(Broker5Digits)
   {
      SL *= 10;
      TP *= 10;
      Slippage *= 10;
      k *= 10;
   }   
      
//----
   start();
   return(0);
}

//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+-----------------------------------------------------------------+
int deinit()
{
//----
   
//----
  for(int i=ObjectsTotal()-1;i>=0;i--)
   if(StringFind(ObjectName(i),"MT")!=-1)
      ObjectDelete(ObjectName(i));
      
//  Comment("");
   return(0);
}

//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
{
   int nOrderTicket = 0;
   int nOrderType = -1;
   int nTradesNumber = 0;
   int nLongs = 0;
   int nShorts = 0;
   int iTicket = 0;
   double sl = 0, tp = 0;

   nLongs = GetLongsNumber();
   nShorts = GetShortsNumber();
   nTradesNumber = nLongs + nShorts;

   for(int x = 0; x < OrdersTotal(); x++)
   {
      OrderSelect(x, SELECT_BY_POS, MODE_TRADES);
      if(OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber)
      {
         nOrderTicket = OrderTicket();
         nOrderType = OrderType();
         break;
      }
   }

   // Close trades
   if(Hour() == NYClose && nTradesNumber > 0)
   {
      CloseAllLongs();
      CloseAllShorts();
      return(0);
   }
   
   // Check Time and do nothing if it is too early or too late
    if(Hour() < RangeEnd)
   {
      Restart = false;
      return(0);
   }
    
   
   
   if(nTradesNumber == 0 && lasttrade < iTime(Symbol(), PERIOD_D1, 0) && Restart == false)
   {
      
      ShowTimeRange();
      int h  =  HH;
      int l  =  LL;
           
      string text = StringConcatenate(" Day:Hour  ",DayOfWeek(),":",Hour(),",  Range High: ", High[h], ", Range Low: ", Low[l]," Lots:",GetLots(SL));
      Comment(text);
      //Comment("Trying...");
      
      //If the price is above the upper boundary of the range go long
      if(Bid >= High[h] + 1*Point)
      {
         if(SL > 0)
            sl = Ask - SL*Point;
         if(TP > 0)
            tp = Ask + TP*Point;
         if(BrokerECN)
         {
            iTicket = OrderSend(Symbol(), OP_BUY, GetLots(SL), Ask, Slippage, 0, 0, "", MagicNumber, 0, Blue);
            if(iTicket > 0)
            {
               OrderModify(iTicket, OrderOpenPrice(), sl, tp, 0, CLR_NONE);
            }
                      
         }
         else
         {
            OrderSend(Symbol(), OP_BUY, GetLots(SL), Ask, Slippage, sl, tp, "", MagicNumber, 0, Blue);
         }
         lasttrade = Time[0];
         SLMoved = 0;
         
         Alert("* Price: ", Ask," SL: ",sl," TP: ",tp);
         Alert("DD:HH:MM  ",DayOfWeek(),":",Hour(),":",Minute()," Buy ", GetLots(SL) ," Lots");
         
         Restart = true;
      }
      //If the price is below the lower boundary of the range go short
      if(Bid <= Low[l] - 1*Point)
      {
         if(SL > 0)
            sl = Bid + SL*Point;
         if(TP > 0)
            tp = Bid - TP*Point;
         if(BrokerECN)
         {
            iTicket = OrderSend(Symbol(), OP_SELL, GetLots(SL), Bid, Slippage, 0, 0, "", MagicNumber, 0, Red);
            if(iTicket > 0)
            {
               OrderModify(iTicket, OrderOpenPrice(), sl, tp, 0, CLR_NONE);
            }
                          
         }
         else
         {
            OrderSend(Symbol(), OP_SELL, GetLots(SL), Bid, Slippage, sl, tp, "", MagicNumber, 0, Red);
         }
         lasttrade = Time[0];
         SLMoved = 0;
         
         Alert("* Price: ", Bid," SL: ",sl," TP: ",tp);
         Alert("DD:HH:MM  ",DayOfWeek(),":",Hour(),":",Minute()," Buy ", GetLots(SL) ," Lots");
         
         Restart = true;
         
      }
   }
   else
   {
      //Comment("not trying");
      //Comment("lasttrade: " +lasttrade);
   }
   
   //Stop updating routine
   if(nTradesNumber > 0)
   {
      OrderSelect(nOrderTicket, SELECT_BY_TICKET, MODE_TRADES);
      if(nLongs > 0)
      {
         //First stop move at +15
         if(SLMoved == 0 && Bid >= OrderOpenPrice() + 15*k*Point)
         {
            OrderModify(nOrderTicket, OrderOpenPrice(), OrderOpenPrice() + k*Point, OrderTakeProfit(), 0, CLR_NONE);
            SLMoved = 1;
            
            Alert("Move Stop @ ",Hour(),":",Minute()," to  ", OrderOpenPrice() + k*Point);
            
         }   
         //Second stop move at +30
         else if(SLMoved == 1 && Bid >= OrderOpenPrice() + 30*k*Point)
         {
            OrderModify(nOrderTicket, OrderOpenPrice(), OrderOpenPrice() + 15*k*Point, OrderTakeProfit(), 0, CLR_NONE);
            SLMoved = 2;
         }   
         //Third stop move at +40
         else if(SLMoved == 2 && Bid >= OrderOpenPrice() + 40*k*Point)
         {
            OrderModify(nOrderTicket, OrderOpenPrice(), OrderOpenPrice() + 30*k*Point, OrderTakeProfit(), 0, CLR_NONE);
            SLMoved = 3;
         }
      }
      if(nShorts > 0)
      {
         //First stop move at +15
         if(SLMoved == 0 && Ask <= OrderOpenPrice() - 15*k*Point)
         {
            OrderModify(nOrderTicket, OrderOpenPrice(), OrderOpenPrice() - k*Point, OrderTakeProfit(), 0, CLR_NONE);
            SLMoved = 1;
            
            Alert("Move Stop @ ",Hour(),":",Minute()," to  ", OrderOpenPrice() - k*Point);
            
         }   
         //Second stop move at +30
         else if(SLMoved == 1 && Ask <= OrderOpenPrice() - 30*k*Point)
         {
            OrderModify(nOrderTicket, OrderOpenPrice(), OrderOpenPrice() - 15*k*Point, OrderTakeProfit(), 0, CLR_NONE);
            SLMoved = 2;
         }   
         //Third stop move at +40
         else if(SLMoved == 2 && Ask <= OrderOpenPrice() - 40*k*Point)
         {
            OrderModify(nOrderTicket, OrderOpenPrice(), OrderOpenPrice() - 30*k*Point, OrderTakeProfit(), 0, CLR_NONE);
            SLMoved = 3;
         }
      }
   }
//----
   return(0);
}

//Just get the number of long positions
int GetLongsNumber()
{
   int n = 0;
   for(int x = 0; x < OrdersTotal(); x++)
   {
      OrderSelect(x, SELECT_BY_POS, MODE_TRADES);
      if(OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber && OrderType() == OP_BUY)
      {
         n++;
      }
   }
   return(n);
}

//Just get the number of short positions
int GetShortsNumber()
{
   int n = 0;
   for(int x = 0; x < OrdersTotal(); x++)
   {
      OrderSelect(x, SELECT_BY_POS, MODE_TRADES);
      if(OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber && OrderType() == OP_SELL)
      {
         n++;
      }
   }
   return(n);
}

//Exit all long positions
void CloseAllLongs()
{
   int total = OrdersTotal();
   for(int i = total - 1; i >= 0; i--)
   {
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
      if(OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber && OrderType() == OP_BUY)
      {
         RefreshRates();
         OrderClose(OrderTicket(), OrderLots(), Bid, Slippage, Blue);
      }
   }
} 
      
//Exit all short positions
void CloseAllShorts()
{
   int total = OrdersTotal();
   for(int i = total - 1; i >= 0; i--)
   {
      OrderSelect(i, SELECT_BY_POS, MODE_TRADES);
      if(OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber && OrderType() == OP_SELL)
      {
         RefreshRates();
         OrderClose(OrderTicket(), OrderLots(), Ask, Slippage, Red);
      }
   }
} 

//Money management. It calculates the lot size getting the Stop Loss size and Risk input
double GetLots(double RiskInPips)
{
   double lot, dif;
   double   MinLot;
   double   MaxLot;
   double   LotStep;
   double   mpl;
   if(UseMoneyManagement)
   {
      double totalrisk = AccountBalance() * Risk / 100;
      double traderisk = totalrisk;
      double pip = traderisk/RiskInPips;
      lot = pip/MarketInfo(Symbol(), MODE_TICKVALUE);
      LotStep = MarketInfo(Symbol(),MODE_LOTSTEP);
      dif = MathMod(lot, LotStep);
      if(dif != 0)
      {
         lot -= dif;
      }
   }
   else
   {
      lot = Lots;
   }
   MinLot = MarketInfo(Symbol(), MODE_MINLOT);
   MaxLot = MarketInfo(Symbol(), MODE_MAXLOT);
   if(lot < MinLot) lot = MinLot;
   if(lot > MaxLot) lot = MaxLot;
   
   return(lot);
}

//+------------------------------------------------------------------+
void ShowTimeRange()
{
   for(int i=0;i<=1440;i++)
      {
         if(
               TimeHour(Time[i])    == RangeEnd  &&
               TimeMinute(Time[i])  == 0
           )
            {
               
               for(int j=i;j<=i+1440;j++)
                  {
                     if(
                           TimeHour(Time[j])    == RangeStart   &&
                           TimeMinute(Time[j])  == 0
                       )
                        {
                           DetectRectangle(j,i);
                           //Comment(i+"/" + j);
                           break;
                        }
                  }
               
               break;
            }
      }
}
//------------------------------------------------------------------------------------------
void DetectRectangle(int StartBar,int EndBar)
{

   HH =  iHighest(Symbol(),0,MODE_HIGH,StartBar-EndBar+1,EndBar);
   LL =  iLowest(Symbol(),0,MODE_LOW,StartBar-EndBar+1,EndBar);
   
   HighestHigh =  High[HH];
   LowestLow   =  Low[LL];
   
   if(ShowRectangle)
      {
         if(ObjectFind(Time[StartBar]+"-"+Time[EndBar])==-1)
            {
               ObjectCreate("MT"+Time[StartBar]+"-"+Time[EndBar],OBJ_RECTANGLE,0,Time[StartBar],High[HH],Time[EndBar],Low[LL]);
               ObjectSet("MT"+Time[StartBar]+"-"+Time[EndBar],OBJPROP_COLOR,Yellow);
               
               WindowRedraw();
            }   
      }
}